#include <stdio.h>

#include "wiringPi.h"

#include "sequence.h"

#define PIN_SIN 25
#define PIN_SCLK 8
#define PIN_XLAT 7

#define PIN_GSCLK 23
#define PIN_BLANK 24
#define PIN_VPRG 18

#define pulse(PIN) digitalWrite(PIN, 1); digitalWrite(PIN, 0);

void initDotCorrection();
void writeFrame();

int main() {
	int rc;

	rc = wiringPiSetupGpio();
	if (rc != 0) {
		printf("Error setting up wiringPi");
		return rc;
	}

	pinMode(PIN_SIN, OUTPUT);
	pinMode(PIN_SCLK, OUTPUT);
	pinMode(PIN_XLAT, OUTPUT);

	pinMode(PIN_GSCLK, OUTPUT);
	pinMode(PIN_BLANK, OUTPUT);
	pinMode(PIN_VPRG, OUTPUT);

	digitalWrite(PIN_SIN, 0);
	digitalWrite(PIN_SCLK, 0);
	digitalWrite(PIN_XLAT, 0);

	digitalWrite(PIN_GSCLK, 0);
	digitalWrite(PIN_BLANK, 1);
	digitalWrite(PIN_VPRG, 0);

	initDotCorrection();

	for (;;) {
		for (int frame = 0; frame < FRAMES; frame++) {
			writeFrame(frames[frame]);
		}
	}

	return 0;
}

void initDotCorrection() {
	digitalWrite(PIN_VPRG, 1);

	digitalWrite(PIN_SIN, 1);

	for (int i = 0; i < 96; i++) {
		pulse(PIN_SCLK);
	}
	pulse(PIN_XLAT);

	digitalWrite(PIN_VPRG, 0);
}

void writeLed();

void writeFrame(int* frame) {
	for (int led = 0; led < LEDS; led++) {
		writeLed(frame[led]);
	}

	pulse(PIN_XLAT);

	for (int pwmLoop = 0; pwmLoop < 1024; pwmLoop++) {
		digitalWrite(PIN_BLANK, 0);
		for (int level = 0; level < 256; level++) { // 8 bit
			pulse(PIN_GSCLK);
		}
		digitalWrite(PIN_BLANK, 1);
	}
}

void writeLed(int value) {
	// "convert" 3 bit to 8, leaving the 4 leading bits zero.
	value = value << 5;

	for (int i = 11; i >= 0; i--) { // MSB first
		int bit = (value >> i) & 0x001;
		digitalWrite(PIN_SIN, bit);
		pulse(PIN_SCLK);
	}
}
